namespace dnnl {
namespace impl {
namespace gpu {
namespace intel {
namespace ocl {
const char *nhwc_reusable_kernel = R"==(/*******************************************************************************)==""\n"
R"==(* Copyright 2020-2024 Intel Corporation)==""\n"
R"==(*)==""\n"
R"==(* Licensed under the Apache License, Version 2.0 (the "License");)==""\n"
R"==(* you may not use this file except in compliance with the License.)==""\n"
R"==(* You may obtain a copy of the License at)==""\n"
R"==(*)==""\n"
R"==(* http:)==""\n"
R"==(*)==""\n"
R"==(* Unless required by applicable law or agreed to in writing, software)==""\n"
R"==(* distributed under the License is distributed on an "AS IS" BASIS,)==""\n"
R"==(* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.)==""\n"
R"==(* See the License for the specific language governing permissions and)==""\n"
R"==(* limitations under the License.)==""\n"
R"==(*******************************************************************************/)==""\n"
R"==(#include "gpu/intel/ocl/bnorm/nhwc_reusable.h")==""\n"
R"==(void nhwc_reusable_1pass_fused_reduction(volatile __global atomic_float *mean,)==""\n"
R"==(volatile __global atomic_float *variance, off_t dst_offset,)==""\n"
R"==(SUM_DATA_T *sum, SUM_DATA_T *sum_sq, __local SUM_DATA_T *local_sum,)==""\n"
R"==(__local SUM_DATA_T *local_sum_sq, off_t vect_size) {)==""\n"
R"==(const int local_id = get_local_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int row_size = vect_size * SUB_GROUP_SIZE;)==""\n"
R"==(const int group_size = get_local_size(1);)==""\n"
R"==(if (local_id > 0) {)==""\n"
R"==(unroll_4_for(int v_idx = 0; v_idx < vect_size; v_idx++) {)==""\n"
R"==(const int slm_offset)==""\n"
R"==(= local_id * row_size + v_idx * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(local_sum[slm_offset] = sum[v_idx];)==""\n"
R"==(local_sum_sq[slm_offset] = sum_sq[v_idx];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (local_id == 0) {)==""\n"
R"==(unroll_16_for(int l_id = 1; l_id < group_size; l_id++) {)==""\n"
R"==(for (int v_idx = 0; v_idx < vect_size; v_idx++) {)==""\n"
R"==(const int off)==""\n"
R"==(= l_id * row_size + v_idx * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(SUM_DATA_T tmp = local_sum[off];)==""\n"
R"==(SUM_DATA_T tmp_sq = local_sum_sq[off];)==""\n"
R"==(sum[v_idx] = summation(tmp.s1, sum[v_idx]);)==""\n"
R"==(sum_sq[v_idx] = summation(tmp_sq.s1, sum_sq[v_idx]);)==""\n"
R"==(sum[v_idx] = summation(tmp.s0, sum[v_idx]);)==""\n"
R"==(sum_sq[v_idx] = summation(tmp_sq.s0, sum_sq[v_idx]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(unroll_4_for(int v_idx = 0; v_idx < vect_size; v_idx++) {)==""\n"
R"==(const int off = v_idx * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(atomic_add_global(&mean[dst_offset + off], sum[v_idx].s0);)==""\n"
R"==(atomic_add_global(&variance[dst_offset + off], sum_sq[v_idx].s0);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(return;)==""\n"
R"==(})==""\n"
R"==(void nhwc_reusable_1pass_fused_reduction_buff()==""\n"
R"==(volatile __global atomic_float *mean,)==""\n"
R"==(volatile __global atomic_float *variance, off_t dst_offset,)==""\n"
R"==(SUM_DATA_T *sum, SUM_DATA_T *sum_sq, __local SUM_DATA_T *local_sum,)==""\n"
R"==(__local SUM_DATA_T *local_sum_sq, off_t ic_block) {)==""\n"
R"==(const int local_id = get_local_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int row_size = ic_block;)==""\n"
R"==(const int group_size = get_local_size(1);)==""\n"
R"==(const int ic_block_sgroups = ic_block / SUB_GROUP_SIZE;)==""\n"
R"==(if (local_id > 0) {)==""\n"
R"==(unroll_16_for(int sg = 0; sg < ic_block_sgroups; sg++) {)==""\n"
R"==(const int slm_offset)==""\n"
R"==(= local_id * row_size + sg * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(local_sum[slm_offset] = sum[sg];)==""\n"
R"==(local_sum_sq[slm_offset] = sum_sq[sg];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (local_id == 0) {)==""\n"
R"==(unroll_16_for(int l_id = 1; l_id < group_size; l_id++) {)==""\n"
R"==(unroll_4_for(int sg = 0; sg < ic_block_sgroups; sg++) {)==""\n"
R"==(const int off = l_id * row_size + sg * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(SUM_DATA_T tmp = local_sum[off];)==""\n"
R"==(SUM_DATA_T tmp_sq = local_sum_sq[off];)==""\n"
R"==(sum[sg] = summation(tmp.s1, sum[sg]);)==""\n"
R"==(sum_sq[sg] = summation(tmp_sq.s1, sum_sq[sg]);)==""\n"
R"==(sum[sg] = summation(tmp.s0, sum[sg]);)==""\n"
R"==(sum_sq[sg] = summation(tmp_sq.s0, sum_sq[sg]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(unroll_4_for(int sg = 0; sg < ic_block_sgroups; sg++) {)==""\n"
R"==(const int off = sg * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(atomic_add_global(&mean[dst_offset + off], sum[sg].s0);)==""\n"
R"==(atomic_add_global(&variance[dst_offset + off], sum_sq[sg].s0);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(return;)==""\n"
R"==(})==""\n"
R"==(void nhwc_reusable_reg_fused_reduction(volatile __global atomic_float *dst,)==""\n"
R"==(off_t dst_offset, float *sum, __local float *local_sum,)==""\n"
R"==(off_t vect_size) {)==""\n"
R"==(const int local_id = get_local_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int row_size = vect_size * SUB_GROUP_SIZE;)==""\n"
R"==(const int group_size = get_local_size(1);)==""\n"
R"==(if (local_id > 0) {)==""\n"
R"==(unroll_4_for(int v_idx = 0; v_idx < vect_size; v_idx++) {)==""\n"
R"==(const int slm_offset)==""\n"
R"==(= local_id * row_size + v_idx * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(local_sum[slm_offset] = sum[v_idx];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (local_id == 0) {)==""\n"
R"==(unroll_16_for(int l_id = 1; l_id < group_size; l_id++) {)==""\n"
R"==(for (int v_idx = 0; v_idx < vect_size; v_idx++) {)==""\n"
R"==(const int off)==""\n"
R"==(= l_id * row_size + v_idx * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(sum[v_idx] += local_sum[off];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(unroll_4_for(int v_idx = 0; v_idx < vect_size; v_idx++) {)==""\n"
R"==(const int off = v_idx * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(atomic_add_global(&dst[dst_offset + off], sum[v_idx]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(return;)==""\n"
R"==(})==""\n"
R"==(void nhwc_reusable_reg_fused_reduction_buff(volatile __global atomic_float *dst,)==""\n"
R"==(off_t dst_offset, float *sum, __local float *local_sum,)==""\n"
R"==(off_t ic_block) {)==""\n"
R"==(const int local_id = get_local_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int group_size = get_local_size(1);)==""\n"
R"==(const int row_size = ic_block;)==""\n"
R"==(const int ic_block_sgroups = ic_block / SUB_GROUP_SIZE;)==""\n"
R"==(if (local_id > 0) {)==""\n"
R"==(unroll_16_for(int sg = 0; sg < ic_block_sgroups; sg++) {)==""\n"
R"==(const int slm_offset)==""\n"
R"==(= local_id * row_size + sg * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(local_sum[slm_offset] = sum[sg];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (local_id == 0) {)==""\n"
R"==(unroll_16_for(int l_id = 1; l_id < group_size; l_id++) {)==""\n"
R"==(unroll_4_for(int sg = 0; sg < ic_block_sgroups; sg++) {)==""\n"
R"==(const int off = l_id * row_size + sg * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(sum[sg] += local_sum[off];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(unroll_4_for(int sg = 0; sg < ic_block_sgroups; sg++) {)==""\n"
R"==(const int off = sg * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(atomic_add_global(&dst[dst_offset + off], sum[sg]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(return;)==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_calc_mean(__global DATA_T *src, __global float *reduce_temp,)==""\n"
R"==(volatile __global atomic_float *mean, off_t ic_size, off_t ic_block,)==""\n"
R"==(off_t sp_size, off_t stat_sp_block, off_t reduce_stat_nblocks,)==""\n"
R"==(int use_fused_atomics_reduction, __local float *local_sum) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(const int src_off)==""\n"
R"==(= ic_block_offset + sp_block_idx * stat_sp_block * ic_size;)==""\n"
R"==(const int reduce_off = ic_block_offset + sp_block_idx * ic_size;)==""\n"
R"==(src += src_off;)==""\n"
R"==(reduce_temp += reduce_off;)==""\n"
R"==(const int sp_idx_bnd = sp_size % stat_sp_block)==""\n"
R"==(? min(stat_sp_block, sp_size - sp_block_idx * stat_sp_block))==""\n"
R"==(: stat_sp_block;)==""\n"
R"==(const int ic_block_sgroups = ic_block / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups - ic_tail_sgroups;)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups / VECT_SIZE; ++sg) {)==""\n"
R"==(VECT_FLOAT_T v_mean = 0.0f;)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(v_mean += LOAD_VECT_DATA()==""\n"
R"==(&src[sg * SUB_GROUP_SIZE * VECT_SIZE + sp * ic_size]);)==""\n"
R"==(})==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(const int dst_off)==""\n"
R"==(= ic_block_offset + sg * VECT_SIZE * SUB_GROUP_SIZE;)==""\n"
R"==(nhwc_reusable_reg_fused_reduction()==""\n"
R"==(mean, dst_off, (float *)(&v_mean), local_sum, VECT_SIZE);)==""\n"
R"==(} else {)==""\n"
R"==(const int sg_off = sg * VECT_SIZE * SUB_GROUP_SIZE;)==""\n"
R"==(for (int v_idx = 0; v_idx < VECT_SIZE; v_idx++) {)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[sg_off + v_idx * SUB_GROUP_SIZE],)==""\n"
R"==(#if VECT_SIZE > 1)==""\n"
R"==(v_mean[v_idx]);)==""\n"
R"==(#else)==""\n"
R"==(v_mean);)==""\n"
R"==(#endif)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(float v_mean = 0.0f;)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(v_mean += LOAD_DATA_1x16()==""\n"
R"==(&src[(ic_vect_sgroups + sg) * SUB_GROUP_SIZE)==""\n"
R"==(+ sp * ic_size]);)==""\n"
R"==(})==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(const int dst_off)==""\n"
R"==(= ic_block_offset + (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(nhwc_reusable_reg_fused_reduction()==""\n"
R"==(mean, dst_off, &v_mean, local_sum, 1);)==""\n"
R"==(} else {)==""\n"
R"==(const int sg_off = (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[sg_off], v_mean);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_calc_mean_buff(__global DATA_T *src, __global float *reduce_temp,)==""\n"
R"==(volatile __global atomic_float *mean, off_t ic_size, off_t ic_block,)==""\n"
R"==(off_t sp_size, off_t stat_sp_block, off_t reduce_stat_nblocks,)==""\n"
R"==(int use_fused_atomics_reduction, __local float *local_sum) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(const int src_off)==""\n"
R"==(= ic_block_offset + sp_block_idx * stat_sp_block * ic_size;)==""\n"
R"==(const int reduce_off = ic_block_offset + sp_block_idx * ic_size;)==""\n"
R"==(src += src_off;)==""\n"
R"==(reduce_temp += reduce_off;)==""\n"
R"==(const int sp_idx_bnd = sp_size % stat_sp_block)==""\n"
R"==(? min(stat_sp_block, sp_size - sp_block_idx * stat_sp_block))==""\n"
R"==(: stat_sp_block;)==""\n"
R"==(const int ic_block_sgroups)==""\n"
R"==(= min(ic_size - ic_block_offset, ic_block) / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups / VECT_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(float v_mean[MAX_IC_BLOCK_SGROUPS] = {0.0f};)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(for (int sg = 0; sg < ic_vect_sgroups; ++sg) {)==""\n"
R"==(float s_vect[VECT_SIZE];)==""\n"
R"==(AS_VECT_FLOAT(s_vect))==""\n"
R"==(= LOAD_VECT_DATA(&src[sg * SUB_GROUP_SIZE * VECT_SIZE]);)==""\n"
R"==(for (int vect = 0; vect < VECT_SIZE; ++vect) {)==""\n"
R"==(v_mean[sg * VECT_SIZE + vect] += s_vect[vect];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(#if MAY_HAVE_IC_TAIL)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = ic_vect_sgroups * VECT_SIZE + sg;)==""\n"
R"==(v_mean[sg_idx] += LOAD_DATA_1x16(&src[sg_idx * SUB_GROUP_SIZE]);)==""\n"
R"==(})==""\n"
R"==(#endif )==""\n"
R"==(src += ic_size;)==""\n"
R"==(} )==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(nhwc_reusable_reg_fused_reduction_buff()==""\n"
R"==(mean, ic_block_offset, (float *)(&v_mean), local_sum, ic_block);)==""\n"
R"==(} else {)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups; ++sg) {)==""\n"
R"==(const int sg_off = sg * SUB_GROUP_SIZE;)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[sg_off], v_mean[sg]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_calc_var(__global DATA_T *src, __global float *mean,)==""\n"
R"==(__global float *reduce_temp, volatile __global atomic_float *variance,)==""\n"
R"==(off_t ic_size, off_t ic_block, off_t sp_size, off_t stat_sp_block,)==""\n"
R"==(off_t reduce_stat_nblocks, int use_fused_atomics_reduction,)==""\n"
R"==(__local float *local_sum) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(const int src_off)==""\n"
R"==(= ic_block_offset + sp_block_idx * stat_sp_block * ic_size;)==""\n"
R"==(const int reduce_off = ic_block_offset + sp_block_idx * ic_size;)==""\n"
R"==(src += src_off;)==""\n"
R"==(reduce_temp += reduce_off + reduce_stat_nblocks * ic_size;)==""\n"
R"==(mean += ic_block_offset;)==""\n"
R"==(const int sp_idx_bnd = sp_size % stat_sp_block)==""\n"
R"==(? min(stat_sp_block, sp_size - sp_block_idx * stat_sp_block))==""\n"
R"==(: stat_sp_block;)==""\n"
R"==(const int ic_block_sgroups = ic_block / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups - ic_tail_sgroups;)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups / VECT_SIZE; ++sg) {)==""\n"
R"==(VECT_FLOAT_T v_var = 0.0f;)==""\n"
R"==(const VECT_FLOAT_T v_mean)==""\n"
R"==(= LOAD_VECT_FLOAT(&mean[sg * SUB_GROUP_SIZE * VECT_SIZE]);)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(const VECT_FLOAT_T v0)==""\n"
R"==(= LOAD_VECT_DATA(&src[sg * SUB_GROUP_SIZE * VECT_SIZE)==""\n"
R"==(+ sp * ic_size]))==""\n"
R"==(- v_mean;)==""\n"
R"==(v_var = fma(v0, v0, v_var);)==""\n"
R"==(})==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(const int dst_off)==""\n"
R"==(= ic_block_offset + sg * VECT_SIZE * SUB_GROUP_SIZE;)==""\n"
R"==(nhwc_reusable_reg_fused_reduction()==""\n"
R"==(variance, dst_off, (float *)(&v_var), local_sum, VECT_SIZE);)==""\n"
R"==(} else {)==""\n"
R"==(const int sg_off = sg * VECT_SIZE * SUB_GROUP_SIZE;)==""\n"
R"==(for (int v_idx = 0; v_idx < VECT_SIZE; v_idx++) {)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[sg_off + v_idx * SUB_GROUP_SIZE],)==""\n"
R"==(#if VECT_SIZE > 1)==""\n"
R"==(v_var[v_idx]);)==""\n"
R"==(#else)==""\n"
R"==(v_var);)==""\n"
R"==(#endif)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(float v_var = 0.0f;)==""\n"
R"==(const float v_mean = LOAD_FLOAT_1x16()==""\n"
R"==(&mean[(ic_vect_sgroups + sg) * SUB_GROUP_SIZE]);)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(const float v0)==""\n"
R"==(= LOAD_DATA_1x16()==""\n"
R"==(&src[(ic_vect_sgroups + sg) * SUB_GROUP_SIZE)==""\n"
R"==(+ sp * ic_size]))==""\n"
R"==(- v_mean;)==""\n"
R"==(v_var = fma(v0, v0, v_var);)==""\n"
R"==(})==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(const int dst_off)==""\n"
R"==(= ic_block_offset + (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(nhwc_reusable_reg_fused_reduction()==""\n"
R"==(variance, dst_off, &v_var, local_sum, 1);)==""\n"
R"==(} else {)==""\n"
R"==(const int sg_off = (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[sg_off], v_var);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_calc_var_buff(__global DATA_T *src, __global float *mean,)==""\n"
R"==(__global float *reduce_temp, volatile __global atomic_float *variance,)==""\n"
R"==(off_t ic_size, off_t ic_block, off_t sp_size, off_t stat_sp_block,)==""\n"
R"==(off_t reduce_stat_nblocks, int use_fused_atomics_reduction,)==""\n"
R"==(__local float *local_sum) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(const int src_off)==""\n"
R"==(= ic_block_offset + sp_block_idx * stat_sp_block * ic_size;)==""\n"
R"==(const int reduce_off = ic_block_offset + sp_block_idx * ic_size;)==""\n"
R"==(src += src_off;)==""\n"
R"==(reduce_temp += reduce_off + reduce_stat_nblocks * ic_size;)==""\n"
R"==(mean += ic_block_offset;)==""\n"
R"==(const int sp_idx_bnd = sp_size % stat_sp_block)==""\n"
R"==(? min(stat_sp_block, sp_size - sp_block_idx * stat_sp_block))==""\n"
R"==(: stat_sp_block;)==""\n"
R"==(const int ic_block_sgroups)==""\n"
R"==(= min(ic_size - ic_block_offset, ic_block) / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups / VECT_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(float v_mean[MAX_IC_BLOCK_SGROUPS] = {0.0f};)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups; ++sg) {)==""\n"
R"==(v_mean[sg] = as_float(intel_sub_group_block_read()==""\n"
R"==((const __global uint *)(&mean[(sg * SUB_GROUP_SIZE)])));)==""\n"
R"==(})==""\n"
R"==(float v_var[MAX_IC_BLOCK_SGROUPS] = {0.0f};)==""\n"
R"==(float v0[MAX_IC_BLOCK_SGROUPS] = {0.0f};)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(for (int sg = 0; sg < ic_vect_sgroups; ++sg) {)==""\n"
R"==(float s_vect[VECT_SIZE];)==""\n"
R"==(AS_VECT_FLOAT(s_vect))==""\n"
R"==(= LOAD_VECT_DATA(&src[sg * SUB_GROUP_SIZE * VECT_SIZE]);)==""\n"
R"==(for (int vect = 0; vect < VECT_SIZE; ++vect) {)==""\n"
R"==(int sg_idx = sg * VECT_SIZE + vect;)==""\n"
R"==(v0[sg_idx] = s_vect[vect] - v_mean[sg_idx];)==""\n"
R"==(v_var[sg_idx] = fma(v0[sg_idx], v0[sg_idx], v_var[sg_idx]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(#if MAY_HAVE_IC_TAIL)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = ic_vect_sgroups * VECT_SIZE + sg;)==""\n"
R"==(float s_tail = LOAD_DATA_1x16(&src[sg_idx * SUB_GROUP_SIZE]);)==""\n"
R"==(v0[sg_idx] = s_tail - v_mean[sg_idx];)==""\n"
R"==(v_var[sg_idx] = fma(v0[sg_idx], v0[sg_idx], v_var[sg_idx]);)==""\n"
R"==(})==""\n"
R"==(#endif )==""\n"
R"==(src += ic_size;)==""\n"
R"==(} )==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(nhwc_reusable_reg_fused_reduction_buff(variance, ic_block_offset,)==""\n"
R"==((float *)(&v_var), local_sum, ic_block);)==""\n"
R"==(} else {)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups; ++sg) {)==""\n"
R"==(const int sg_off = sg * SUB_GROUP_SIZE;)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[sg_off], v_var[sg]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_calc_mean_var(__global DATA_T *src, __global float *reduce_temp,)==""\n"
R"==(volatile __global atomic_float *mean,)==""\n"
R"==(volatile __global atomic_float *variance, off_t ic_size, off_t ic_block,)==""\n"
R"==(off_t sp_size, off_t stat_sp_block, off_t reduce_stat_nblocks,)==""\n"
R"==(int use_fused_atomics_reduction, __local SUM_DATA_T *local_sum,)==""\n"
R"==(__local SUM_DATA_T *local_sum_sq) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(const int src_off)==""\n"
R"==(= ic_block_offset + sp_block_idx * stat_sp_block * ic_size;)==""\n"
R"==(const int reduce_off = ic_block_offset + sp_block_idx * ic_size;)==""\n"
R"==(const int variance_off = reduce_stat_nblocks * ic_size;)==""\n"
R"==(src += src_off;)==""\n"
R"==(reduce_temp += reduce_off;)==""\n"
R"==(const int sp_idx_bnd = sp_size % stat_sp_block)==""\n"
R"==(? min(stat_sp_block, sp_size - sp_block_idx * stat_sp_block))==""\n"
R"==(: stat_sp_block;)==""\n"
R"==(const int ic_block_sgroups = ic_block / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups - ic_tail_sgroups;)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups / VECT_SIZE; ++sg) {)==""\n"
R"==(SUM_DATA_T sum[VECT_SIZE] = {0.0f};)==""\n"
R"==(SUM_DATA_T sum_sq[VECT_SIZE] = {0.0f};)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(const VECT_FLOAT_T s_vect = LOAD_VECT_DATA()==""\n"
R"==(&src[sg * SUB_GROUP_SIZE * VECT_SIZE + sp * ic_size]);)==""\n"
R"==(for (int v_idx = 0; v_idx < VECT_SIZE; ++v_idx) {)==""\n"
R"==(#if VECT_SIZE > 1)==""\n"
R"==(#define S_VECT s_vect[v_idx])==""\n"
R"==(#else)==""\n"
R"==(#define S_VECT s_vect)==""\n"
R"==(#endif)==""\n"
R"==(sum[v_idx] = summation(S_VECT, sum[v_idx]);)==""\n"
R"==(sum_sq[v_idx] = summation(S_VECT * S_VECT, sum_sq[v_idx]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(const int dst_off)==""\n"
R"==(= ic_block_offset + sg * VECT_SIZE * SUB_GROUP_SIZE;)==""\n"
R"==(nhwc_reusable_1pass_fused_reduction(mean, variance, dst_off, sum,)==""\n"
R"==(sum_sq, local_sum, local_sum_sq, VECT_SIZE);)==""\n"
R"==(} else {)==""\n"
R"==(const int sg_off = sg * VECT_SIZE * SUB_GROUP_SIZE;)==""\n"
R"==(for (int v_idx = 0; v_idx < VECT_SIZE; v_idx++) {)==""\n"
R"==(const int reduce_off = sg_off + v_idx * SUB_GROUP_SIZE;)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[reduce_off], sum[v_idx].s0);)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[variance_off + reduce_off],)==""\n"
R"==(sum_sq[v_idx].s0);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(SUM_DATA_T sum = 0.0f;)==""\n"
R"==(SUM_DATA_T sum_sq = 0.0f;)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(const float src_v = LOAD_DATA_1x16()==""\n"
R"==(&src[(ic_vect_sgroups + sg) * SUB_GROUP_SIZE)==""\n"
R"==(+ sp * ic_size]);)==""\n"
R"==(sum = summation(src_v, sum);)==""\n"
R"==(sum_sq = summation(src_v * src_v, sum_sq);)==""\n"
R"==(})==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(const int dst_off)==""\n"
R"==(= ic_block_offset + (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(nhwc_reusable_1pass_fused_reduction(mean, variance, dst_off, &sum,)==""\n"
R"==(&sum_sq, local_sum, local_sum_sq, 1);)==""\n"
R"==(} else {)==""\n"
R"==(const int sg_off = (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[sg_off], sum.s0);)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[variance_off + sg_off], sum_sq.s0);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_calc_mean_var_buff(__global DATA_T *src,)==""\n"
R"==(__global float *reduce_temp, volatile __global atomic_float *mean,)==""\n"
R"==(volatile __global atomic_float *variance, off_t ic_size, off_t ic_block,)==""\n"
R"==(off_t sp_size, off_t stat_sp_block, off_t reduce_stat_nblocks,)==""\n"
R"==(int use_fused_atomics_reduction, __local SUM_DATA_T *local_sum,)==""\n"
R"==(__local SUM_DATA_T *local_sum_sq) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(const int src_off)==""\n"
R"==(= ic_block_offset + sp_block_idx * stat_sp_block * ic_size;)==""\n"
R"==(const int reduce_off = ic_block_offset + sp_block_idx * ic_size;)==""\n"
R"==(const int variance_off = reduce_stat_nblocks * ic_size;)==""\n"
R"==(src += src_off;)==""\n"
R"==(reduce_temp += reduce_off;)==""\n"
R"==(const int sp_idx_bnd = sp_size % stat_sp_block)==""\n"
R"==(? min(stat_sp_block, sp_size - sp_block_idx * stat_sp_block))==""\n"
R"==(: stat_sp_block;)==""\n"
R"==(const int ic_block_sgroups)==""\n"
R"==(= min(ic_size - ic_block_offset, ic_block) / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups / VECT_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(SUM_DATA_T sum[MAX_IC_BLOCK_SGROUPS] = {0.0f};)==""\n"
R"==(SUM_DATA_T sum_sq[MAX_IC_BLOCK_SGROUPS] = {0.0f};)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(for (int sg = 0; sg < ic_vect_sgroups; ++sg) {)==""\n"
R"==(float s_vect[VECT_SIZE];)==""\n"
R"==(AS_VECT_FLOAT(s_vect))==""\n"
R"==(= LOAD_VECT_DATA(&src[sg * SUB_GROUP_SIZE * VECT_SIZE]);)==""\n"
R"==(for (int vect = 0; vect < VECT_SIZE; ++vect) {)==""\n"
R"==(const int sum_idx = sg * VECT_SIZE + vect;)==""\n"
R"==(sum[sum_idx] = summation(s_vect[vect], sum[sum_idx]);)==""\n"
R"==(sum_sq[sum_idx] = summation()==""\n"
R"==(s_vect[vect] * s_vect[vect], sum_sq[sum_idx]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(#if MAY_HAVE_IC_TAIL)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = ic_vect_sgroups * VECT_SIZE + sg;)==""\n"
R"==(float s_tail = LOAD_DATA_1x16(&src[sg_idx * SUB_GROUP_SIZE]);)==""\n"
R"==(sum[sg_idx] = summation(s_tail, sum[sg_idx]);)==""\n"
R"==(sum_sq[sg_idx] = summation(s_tail * s_tail, sum_sq[sg_idx]);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(src += ic_size;)==""\n"
R"==(})==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(nhwc_reusable_1pass_fused_reduction_buff(mean, variance,)==""\n"
R"==(ic_block_offset, sum, sum_sq, local_sum, local_sum_sq,)==""\n"
R"==(ic_block);)==""\n"
R"==(} else {)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups; ++sg) {)==""\n"
R"==(const int reduce_off = sg * SUB_GROUP_SIZE;)==""\n"
R"==(STORE_FLOAT_1x16(&reduce_temp[reduce_off], sum[sg].s0);)==""\n"
R"==(STORE_FLOAT_1x16()==""\n"
R"==(&reduce_temp[variance_off + reduce_off], sum_sq[sg].s0);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_norm_fwd(__global DATA_T *src, __global float *mean,)==""\n"
R"==(__global float *variance, __global DATA_T *dst,)==""\n"
R"==(__global float *scaleshift, __global float *shift, __global char *ws,)==""\n"
R"==(float eps, __global DATA_T *src_add, float relu_alpha, off_t ic_size,)==""\n"
R"==(off_t ic_block, off_t sp_size, off_t update_sp_block) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp = get_global_id(1) * update_sp_block;)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(mean += ic_block_offset;)==""\n"
R"==(variance += ic_block_offset;)==""\n"
R"==(shift += ic_block_offset;)==""\n"
R"==(scaleshift += ic_block_offset;)==""\n"
R"==(const uint d_off = sp * ic_size + ic_block_offset;)==""\n"
R"==(src += d_off;)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(src_add += d_off;)==""\n"
R"==(#endif)==""\n"
R"==(dst += d_off;)==""\n"
R"==(#if FUSE_BN_RELU && IS_TRAINING)==""\n"
R"==(ws += d_off;)==""\n"
R"==(#endif)==""\n"
R"==(const bool has_sp_block_tail = sp_size % update_sp_block;)==""\n"
R"==(const int sp_idx_bnd = has_sp_block_tail)==""\n"
R"==(? min(update_sp_block, sp_size - sp))==""\n"
R"==(: update_sp_block;)==""\n"
R"==(const int ic_block_sgroups = ic_block / SUB_GROUP_SIZE;)==""\n"
R"==(for (int sp_idx = 0; sp_idx < sp_idx_bnd; sp_idx++) {)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups / VECT_SIZE; ++sg) {)==""\n"
R"==(const int sg_idx = sg * SUB_GROUP_SIZE * VECT_SIZE;)==""\n"
R"==(const VECT_FLOAT_T sm = USE_SCALE)==""\n"
R"==(? LOAD_VECT_FLOAT(&scaleshift[sg_idx]))==""\n"
R"==(: (VECT_FLOAT_T)1.0f;)==""\n"
R"==(const VECT_FLOAT_T sv = USE_SHIFT ? LOAD_VECT_FLOAT(&shift[sg_idx]))==""\n"
R"==(: (VECT_FLOAT_T)0.0f;)==""\n"
R"==(const VECT_FLOAT_T s_vect = LOAD_VECT_DATA(&src[sg_idx]);)==""\n"
R"==(const VECT_FLOAT_T v_mean = LOAD_VECT_FLOAT(&mean[sg_idx]);)==""\n"
R"==(const VECT_FLOAT_T v_variance = LOAD_VECT_FLOAT(&variance[sg_idx]);)==""\n"
R"==(const VECT_FLOAT_T sqrt_variance)==""\n"
R"==(= sm / sqrt(v_variance + (VECT_FLOAT_T)eps);)==""\n"
R"==(VECT_FLOAT_T d_vect = fma(s_vect - v_mean, sqrt_variance, sv);)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(d_vect += LOAD_VECT_DATA(&src_add[sg_idx]);)==""\n"
R"==(#endif)==""\n"
R"==(const VECT_INT_T ws_vect = ISGREATER(d_vect, (VECT_FLOAT_T)0.0f);)==""\n"
R"==(d_vect = select((VECT_FLOAT_T)0.0f, d_vect, ws_vect);)==""\n"
R"==(#if IS_TRAINING)==""\n"
R"==(STORE_VECT_CHAR(&ws[sg_idx], ws_vect);)==""\n"
R"==(#endif )==""\n"
R"==(#endif )==""\n"
R"==(#if WITH_RELU && WITH_LEAKY_RELU)==""\n"
R"==(VECT_INT_T l_vect;)==""\n"
R"==(#endif )==""\n"
R"==(#if WITH_RELU)==""\n"
R"==(#if WITH_LEAKY_RELU)==""\n"
R"==(l_vect = isless(d_vect, 0.0f);)==""\n"
R"==(d_vect = select(d_vect, d_vect * relu_alpha, l_vect);)==""\n"
R"==(#else)==""\n"
R"==(d_vect = max(d_vect, (VECT_FLOAT_T)0.0f);)==""\n"
R"==(#endif )==""\n"
R"==(#endif )==""\n"
R"==(STORE_VECT_DATA(&dst[sg_idx], d_vect);)==""\n"
R"==(} )==""\n"
R"==(const int ic_tail_sgroups = (ic_block / SUB_GROUP_SIZE) % VECT_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups - ic_tail_sgroups;)==""\n"
R"==(const bool has_ic_vect_tail = ic_tail_sgroups > 0;)==""\n"
R"==(if (has_ic_vect_tail) {)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(const float sm_tail = USE_SCALE)==""\n"
R"==(? LOAD_FLOAT_1x16(&scaleshift[sg_idx]))==""\n"
R"==(: 1.0f;)==""\n"
R"==(const float sv_tail)==""\n"
R"==(= USE_SHIFT ? LOAD_FLOAT_1x16(&shift[sg_idx]) : 0.0f;)==""\n"
R"==(const float v_mean_tail = LOAD_FLOAT_1x16(&mean[sg_idx]);)==""\n"
R"==(const float v_variance_tail)==""\n"
R"==(= LOAD_FLOAT_1x16(&variance[sg_idx]);)==""\n"
R"==(const float sqrt_variance_tail)==""\n"
R"==(= sm_tail / sqrt(v_variance_tail + eps);)==""\n"
R"==(const float s_tail = LOAD_DATA_1x16(&src[sg_idx]);)==""\n"
R"==(float d_tail = fma()==""\n"
R"==(s_tail - v_mean_tail, sqrt_variance_tail, sv_tail);)==""\n"
R"==(if (FUSE_BN_ADD_RELU))==""\n"
R"==(d_tail += LOAD_DATA_1x16(&src_add[sg_idx]);)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(if (d_tail <= 0) d_tail = 0.0f;)==""\n"
R"==(#if IS_TRAINING)==""\n"
R"==(const int ws_tail = d_tail > 0.0f ? -1 : 0;)==""\n"
R"==(STORE_CHAR_1x16(&ws[sg_idx], convert_char(ws_tail));)==""\n"
R"==(#endif )==""\n"
R"==(#endif )==""\n"
R"==(#if WITH_RELU)==""\n"
R"==(#if WITH_LEAKY_RELU)==""\n"
R"==(if (d_tail < 0) d_tail *= relu_alpha;)==""\n"
R"==(#else)==""\n"
R"==(d_tail = max(d_tail, 0.0f);)==""\n"
R"==(#endif )==""\n"
R"==(#endif )==""\n"
R"==(STORE_DATA_1x16(&dst[sg_idx], d_tail);)==""\n"
R"==(})==""\n"
R"==(} )==""\n"
R"==(src += ic_size;)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(src_add += ic_size;)==""\n"
R"==(#endif)==""\n"
R"==(dst += ic_size;)==""\n"
R"==(#if FUSE_BN_RELU && IS_TRAINING)==""\n"
R"==(ws += ic_size;)==""\n"
R"==(#endif)==""\n"
R"==(} )==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_norm_fwd_buff(__global DATA_T *src, __global float *mean,)==""\n"
R"==(__global float *variance, __global DATA_T *dst,)==""\n"
R"==(__global float *scaleshift, __global float *shift, __global char *ws,)==""\n"
R"==(float eps, __global DATA_T *src_add, float relu_alpha, off_t ic_size,)==""\n"
R"==(off_t ic_block, off_t sp_size, off_t update_sp_block) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp = get_global_id(1) * update_sp_block;)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(mean += ic_block_offset;)==""\n"
R"==(variance += ic_block_offset;)==""\n"
R"==(shift += ic_block_offset;)==""\n"
R"==(scaleshift += ic_block_offset;)==""\n"
R"==(const uint d_off = sp * ic_size + ic_block_offset;)==""\n"
R"==(src += d_off;)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(src_add += d_off;)==""\n"
R"==(#endif)==""\n"
R"==(dst += d_off;)==""\n"
R"==(#if FUSE_BN_RELU && IS_TRAINING)==""\n"
R"==(ws += d_off;)==""\n"
R"==(#endif)==""\n"
R"==(float sm[MAX_IC_BLOCK_SGROUPS], sv[MAX_IC_BLOCK_SGROUPS],)==""\n"
R"==(v_mean[MAX_IC_BLOCK_SGROUPS], v_variance[MAX_IC_BLOCK_SGROUPS],)==""\n"
R"==(sqrt_variance[MAX_IC_BLOCK_SGROUPS];)==""\n"
R"==(const bool has_sp_block_tail = sp_size % update_sp_block;)==""\n"
R"==(const int sp_idx_bnd = has_sp_block_tail)==""\n"
R"==(? min(update_sp_block, sp_size - sp))==""\n"
R"==(: update_sp_block;)==""\n"
R"==(const int ic_block_sgroups)==""\n"
R"==(= min(ic_size - ic_block_offset, ic_block) / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups / VECT_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(const bool has_ic_vect_tail = ic_tail_sgroups > 0;)==""\n"
R"==(for (int sg = 0; sg < ic_vect_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = sg * SUB_GROUP_SIZE * VECT_SIZE;)==""\n"
R"==(const int sgv = sg * VECT_SIZE;)==""\n"
R"==(AS_VECT_FLOAT(&sm[sgv]) = USE_SCALE)==""\n"
R"==(? LOAD_VECT_FLOAT(&scaleshift[sg_idx]))==""\n"
R"==(: (VECT_FLOAT_T)1.0f;)==""\n"
R"==(AS_VECT_FLOAT(&sv[sgv]) = USE_SHIFT ? LOAD_VECT_FLOAT(&shift[sg_idx]))==""\n"
R"==(: (VECT_FLOAT_T)0.0f;)==""\n"
R"==(AS_VECT_FLOAT(&v_mean[sgv]) = LOAD_VECT_FLOAT(&mean[sg_idx]);)==""\n"
R"==(AS_VECT_FLOAT(&v_variance[sgv]) = LOAD_VECT_FLOAT(&variance[sg_idx]);)==""\n"
R"==(AS_VECT_FLOAT(&sqrt_variance[sgv]) = AS_VECT_FLOAT(&sm[sgv]))==""\n"
R"==(/ sqrt(AS_VECT_FLOAT(&v_variance[sgv]) + (VECT_FLOAT_T)eps);)==""\n"
R"==(})==""\n"
R"==(#if MAY_HAVE_IC_TAIL)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sgv = ic_vect_sgroups * VECT_SIZE + sg;)==""\n"
R"==(const int sg_idx = (ic_vect_sgroups * VECT_SIZE + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(sm[sgv] = USE_SCALE ? LOAD_FLOAT_1x16(&scaleshift[sg_idx]) : 1.0f;)==""\n"
R"==(sv[sgv] = USE_SHIFT ? LOAD_FLOAT_1x16(&shift[sg_idx]) : 0.0f;)==""\n"
R"==(v_mean[sgv] = LOAD_FLOAT_1x16(&mean[sg_idx]);)==""\n"
R"==(v_variance[sgv] = LOAD_FLOAT_1x16(&variance[sg_idx]);)==""\n"
R"==(sqrt_variance[sgv] = sm[sgv] / sqrt(v_variance[sgv] + eps);)==""\n"
R"==(})==""\n"
R"==(#endif )==""\n"
R"==(for (int sp_idx = 0; sp_idx < sp_idx_bnd; sp_idx++) {)==""\n"
R"==(for (int sg = 0; sg < ic_vect_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = sg * SUB_GROUP_SIZE * VECT_SIZE;)==""\n"
R"==(const int sgv = sg * VECT_SIZE;)==""\n"
R"==(VECT_FLOAT_T d_vect;)==""\n"
R"==(const VECT_FLOAT_T s_vect = LOAD_VECT_DATA(&src[sg_idx]);)==""\n"
R"==(d_vect = fma(s_vect - AS_VECT_FLOAT(&v_mean[sgv]),)==""\n"
R"==(AS_VECT_FLOAT(&sqrt_variance[sgv]),)==""\n"
R"==(AS_VECT_FLOAT(&sv[sgv]));)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(d_vect += LOAD_VECT_DATA(&src_add[sg_idx]);)==""\n"
R"==(#endif)==""\n"
R"==(const VECT_INT_T ws_vect = ISGREATER(d_vect, (VECT_FLOAT_T)0.0f);)==""\n"
R"==(d_vect = select((VECT_FLOAT_T)0.0f, d_vect, ws_vect);)==""\n"
R"==(#if IS_TRAINING)==""\n"
R"==(STORE_VECT_CHAR(&ws[sg_idx], ws_vect);)==""\n"
R"==(#endif )==""\n"
R"==(#endif )==""\n"
R"==(#if WITH_RELU && WITH_LEAKY_RELU)==""\n"
R"==(VECT_INT_T l_vect;)==""\n"
R"==(#endif )==""\n"
R"==(#if WITH_RELU)==""\n"
R"==(#if WITH_LEAKY_RELU)==""\n"
R"==(l_vect = isless(d_vect, 0.0f);)==""\n"
R"==(d_vect = select(d_vect, d_vect * relu_alpha, l_vect);)==""\n"
R"==(#else)==""\n"
R"==(d_vect = max(d_vect, (VECT_FLOAT_T)0.0f);)==""\n"
R"==(#endif )==""\n"
R"==(#endif )==""\n"
R"==(STORE_VECT_DATA(&dst[sg_idx], d_vect);)==""\n"
R"==(} )==""\n"
R"==(#if MAY_HAVE_IC_TAIL)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sgv = ic_vect_sgroups * VECT_SIZE + sg;)==""\n"
R"==(const int sg_idx)==""\n"
R"==(= (ic_vect_sgroups * VECT_SIZE + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(float d_tail;)==""\n"
R"==(const float s_tail = LOAD_DATA_1x16(&src[sg_idx]);)==""\n"
R"==(d_tail = fma(s_tail - v_mean[sgv], sqrt_variance[sgv], sv[sgv]);)==""\n"
R"==(if (FUSE_BN_ADD_RELU) d_tail += LOAD_DATA_1x16(&src_add[sg_idx]);)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(if (d_tail <= 0) d_tail = 0.0f;)==""\n"
R"==(#if IS_TRAINING)==""\n"
R"==(const int ws_tail = d_tail > 0.0f ? -1 : 0;)==""\n"
R"==(STORE_CHAR_1x16(&ws[sg_idx], convert_char(ws_tail));)==""\n"
R"==(#endif )==""\n"
R"==(#endif )==""\n"
R"==(#if WITH_RELU)==""\n"
R"==(#if WITH_LEAKY_RELU)==""\n"
R"==(if (d_tail < 0) d_tail *= relu_alpha;)==""\n"
R"==(#else)==""\n"
R"==(d_tail = max(d_tail, 0.0f);)==""\n"
R"==(#endif )==""\n"
R"==(#endif )==""\n"
R"==(STORE_DATA_1x16(&dst[sg_idx], d_tail);)==""\n"
R"==(})==""\n"
R"==(#endif )==""\n"
R"==(src += ic_size;)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(src_add += ic_size;)==""\n"
R"==(#endif)==""\n"
R"==(dst += ic_size;)==""\n"
R"==(#if FUSE_BN_RELU && IS_TRAINING)==""\n"
R"==(ws += ic_size;)==""\n"
R"==(#endif)==""\n"
R"==(} )==""\n"
R"==(})==""\n"
R"==(void nhwc_reusable_bwd_fused_reduction()==""\n"
R"==(volatile __global atomic_float *diff_scale,)==""\n"
R"==(volatile __global atomic_float *diff_shift, off_t dst_offset,)==""\n"
R"==(float *diff_gamma, float *diff_beta, __local float *local_sums,)==""\n"
R"==(off_t vect_size, off_t calc_slm_size) {)==""\n"
R"==(const int local_id = get_local_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int row_size = vect_size * SUB_GROUP_SIZE;)==""\n"
R"==(const int group_size = get_local_size(1);)==""\n"
R"==(__local float *local_gamma = local_sums;)==""\n"
R"==(__local float *local_beta = local_sums + calc_slm_size / sizeof(float);)==""\n"
R"==(if (local_id > 0) {)==""\n"
R"==(unroll_4_for(int v_idx = 0; v_idx < vect_size; v_idx++) {)==""\n"
R"==(const int slm_offset)==""\n"
R"==(= local_id * row_size + v_idx * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(local_gamma[slm_offset] = diff_gamma[v_idx];)==""\n"
R"==(local_beta[slm_offset] = diff_beta[v_idx];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (local_id == 0) {)==""\n"
R"==(unroll_16_for(int l_id = 1; l_id < group_size; l_id++) {)==""\n"
R"==(for (int v_idx = 0; v_idx < vect_size; v_idx++) {)==""\n"
R"==(const int off)==""\n"
R"==(= l_id * row_size + v_idx * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(diff_gamma[v_idx] += local_gamma[off];)==""\n"
R"==(diff_beta[v_idx] += local_beta[off];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(unroll_4_for(int v_idx = 0; v_idx < vect_size; v_idx++) {)==""\n"
R"==(const int off = v_idx * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(atomic_add_global(&diff_scale[dst_offset + off], diff_gamma[v_idx]);)==""\n"
R"==(atomic_add_global(&diff_shift[dst_offset + off], diff_beta[v_idx]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(return;)==""\n"
R"==(})==""\n"
R"==(void nhwc_reusable_bwd_fused_reduction_buff()==""\n"
R"==(volatile __global atomic_float *diff_scale,)==""\n"
R"==(volatile __global atomic_float *diff_shift, off_t dst_offset,)==""\n"
R"==(float *diff_gamma, float *diff_beta, __local float *local_sums,)==""\n"
R"==(off_t ic_block, off_t calc_slm_size) {)==""\n"
R"==(const int local_id = get_local_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int row_size = ic_block;)==""\n"
R"==(const int group_size = get_local_size(1);)==""\n"
R"==(const int ic_block_sgroups = ic_block / SUB_GROUP_SIZE;)==""\n"
R"==(__local float *local_gamma = local_sums;)==""\n"
R"==(__local float *local_beta = local_sums + calc_slm_size / sizeof(float);)==""\n"
R"==(if (local_id > 0) {)==""\n"
R"==(unroll_16_for(int sg = 0; sg < ic_block_sgroups; sg++) {)==""\n"
R"==(const int slm_offset)==""\n"
R"==(= local_id * row_size + sg * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(local_gamma[slm_offset] = diff_gamma[sg];)==""\n"
R"==(local_beta[slm_offset] = diff_beta[sg];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (local_id == 0) {)==""\n"
R"==(unroll_16_for(int l_id = 1; l_id < group_size; l_id++) {)==""\n"
R"==(unroll_4_for(int sg = 0; sg < ic_block_sgroups; sg++) {)==""\n"
R"==(const int off = l_id * row_size + sg * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(diff_gamma[sg] += local_gamma[off];)==""\n"
R"==(diff_beta[sg] += local_beta[off];)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(unroll_4_for(int sg = 0; sg < ic_block_sgroups; sg++) {)==""\n"
R"==(const int off = sg * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(atomic_add_global(&diff_scale[dst_offset + off], diff_gamma[sg]);)==""\n"
R"==(atomic_add_global(&diff_shift[dst_offset + off], diff_beta[sg]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(return;)==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_calc_stat(__global DATA_T *src, __global float *mean,)==""\n"
R"==(__global DATA_T *diff_dst, __global char *ws,)==""\n"
R"==(__global float *temp_reduce, __global float *temp_reduce_shift,)==""\n"
R"==(volatile __global atomic_float *diff_scale,)==""\n"
R"==(volatile __global atomic_float *diff_shift, off_t ic_size,)==""\n"
R"==(off_t ic_block, off_t sp_size, off_t stat_sp_block,)==""\n"
R"==(off_t reduce_stat_nblocks, int use_fused_atomics_reduction,)==""\n"
R"==(__local float *local_sums, off_t calc_slm_size) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(const int offset = ic_block_offset + sp_block_idx * stat_sp_block * ic_size;)==""\n"
R"==(mean += ic_block_offset;)==""\n"
R"==(src += offset;)==""\n"
R"==(diff_dst += offset;)==""\n"
R"==(ws += offset;)==""\n"
R"==(const int reduce_off = ic_block_offset + (sp_block_idx + 1) * ic_size;)==""\n"
R"==(temp_reduce += reduce_off;)==""\n"
R"==(temp_reduce_shift += reduce_off;)==""\n"
R"==(const bool has_sp_block_tail = sp_size % stat_sp_block;)==""\n"
R"==(const int sp_idx_bnd = has_sp_block_tail)==""\n"
R"==(? min(stat_sp_block, sp_size - sp_block_idx * stat_sp_block))==""\n"
R"==(: stat_sp_block;)==""\n"
R"==(const int ic_block_sgroups = ic_block / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups - ic_tail_sgroups;)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups / VECT_SIZE; ++sg) {)==""\n"
R"==(const int sg_idx = sg * SUB_GROUP_SIZE * VECT_SIZE;)==""\n"
R"==(VECT_FLOAT_T diff_gamma = 0.0f;)==""\n"
R"==(VECT_FLOAT_T diff_beta = 0.0f;)==""\n"
R"==(const VECT_FLOAT_T v_mean = LOAD_VECT_FLOAT(&mean[(sg_idx)]);)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(const int tn_idx = sg_idx + sp * ic_size;)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(const VECT_CHAR_T ws_vect = LOAD_VECT_CHAR(&ws[tn_idx]);)==""\n"
R"==(#endif)==""\n"
R"==(const VECT_FLOAT_T src_vect = LOAD_VECT_DATA(&src[tn_idx]);)==""\n"
R"==(VECT_FLOAT_T dd_vect = LOAD_VECT_DATA(&diff_dst[tn_idx]);)==""\n"
R"==(const VECT_FLOAT_T v0 = src_vect - v_mean;)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(dd_vect = select()==""\n"
R"==((VECT_FLOAT_T)0.0f, dd_vect, CONVERT_VECT_INT_T(ws_vect));)==""\n"
R"==(#endif)==""\n"
R"==(diff_gamma = fma(v0, dd_vect, diff_gamma);)==""\n"
R"==(diff_beta += dd_vect;)==""\n"
R"==(} )==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(const int dst_off)==""\n"
R"==(= ic_block_offset + sg * VECT_SIZE * SUB_GROUP_SIZE;)==""\n"
R"==(nhwc_reusable_bwd_fused_reduction(diff_scale, diff_shift, dst_off,)==""\n"
R"==((float *)(&diff_gamma), (float *)(&diff_beta), local_sums,)==""\n"
R"==(VECT_SIZE, calc_slm_size);)==""\n"
R"==(} else {)==""\n"
R"==(const int sg_off = sg * VECT_SIZE * SUB_GROUP_SIZE;)==""\n"
R"==(for (int v_idx = 0; v_idx < VECT_SIZE; v_idx++) {)==""\n"
R"==(STORE_FLOAT_1x16(&temp_reduce[sg_off + v_idx * SUB_GROUP_SIZE],)==""\n"
R"==(#if VECT_SIZE > 1)==""\n"
R"==(diff_gamma[v_idx]);)==""\n"
R"==(#else)==""\n"
R"==(diff_gamma);)==""\n"
R"==(#endif)==""\n"
R"==(STORE_FLOAT_1x16()==""\n"
R"==(&temp_reduce_shift[sg_off + v_idx * SUB_GROUP_SIZE],)==""\n"
R"==(#if VECT_SIZE > 1)==""\n"
R"==(diff_beta[v_idx]);)==""\n"
R"==(#else)==""\n"
R"==(diff_beta);)==""\n"
R"==(#endif)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(} )==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(float diff_gamma = 0.0f;)==""\n"
R"==(float diff_beta = 0.0f;)==""\n"
R"==(const float v_mean = LOAD_FLOAT_1x16(&mean[(sg_idx)]);)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(const int tn_idx = sg_idx + sp * ic_size;)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(const char ws_vect = LOAD_CHAR_1x16(&ws[tn_idx]);)==""\n"
R"==(#endif)==""\n"
R"==(const float src_vect = LOAD_DATA_1x16(&src[tn_idx]);)==""\n"
R"==(float dd_vect = LOAD_DATA_1x16(&diff_dst[tn_idx]);)==""\n"
R"==(const float v0 = src_vect - v_mean;)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(dd_vect = select(0.0f, dd_vect, convert_int(ws_vect));)==""\n"
R"==(#endif)==""\n"
R"==(diff_gamma = fma(v0, dd_vect, diff_gamma);)==""\n"
R"==(diff_beta += dd_vect;)==""\n"
R"==(} )==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(const int dst_off)==""\n"
R"==(= ic_block_offset + (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(nhwc_reusable_bwd_fused_reduction(diff_scale, diff_shift, dst_off,)==""\n"
R"==((float *)(&diff_gamma), (float *)(&diff_beta), local_sums,)==""\n"
R"==(1, calc_slm_size);)==""\n"
R"==(} else {)==""\n"
R"==(const int sg_off = (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(STORE_FLOAT_1x16(&temp_reduce[sg_off], diff_gamma);)==""\n"
R"==(STORE_FLOAT_1x16(&temp_reduce_shift[sg_off], diff_beta);)==""\n"
R"==(})==""\n"
R"==(} )==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_calc_stat_buff(__global DATA_T *src, __global float *mean,)==""\n"
R"==(__global DATA_T *diff_dst, __global char *ws,)==""\n"
R"==(__global float *temp_reduce, __global float *temp_reduce_shift,)==""\n"
R"==(volatile __global atomic_float *diff_scale,)==""\n"
R"==(volatile __global atomic_float *diff_shift, off_t ic_size,)==""\n"
R"==(off_t ic_block, off_t sp_size, off_t stat_sp_block,)==""\n"
R"==(off_t reduce_stat_nblocks, int use_fused_atomics_reduction,)==""\n"
R"==(__local float *local_sums, off_t calc_slm_size) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(const int offset = ic_block_offset + sp_block_idx * stat_sp_block * ic_size;)==""\n"
R"==(mean += ic_block_offset;)==""\n"
R"==(src += offset;)==""\n"
R"==(diff_dst += offset;)==""\n"
R"==(ws += offset;)==""\n"
R"==(const int reduce_off = ic_block_offset + (sp_block_idx + 1) * ic_size;)==""\n"
R"==(temp_reduce += reduce_off;)==""\n"
R"==(temp_reduce_shift += reduce_off;)==""\n"
R"==(const bool has_sp_block_tail = sp_size % stat_sp_block;)==""\n"
R"==(const int sp_idx_bnd = has_sp_block_tail)==""\n"
R"==(? min(stat_sp_block, sp_size - sp_block_idx * stat_sp_block))==""\n"
R"==(: stat_sp_block;)==""\n"
R"==(const int ic_block_sgroups)==""\n"
R"==(= min(ic_size - ic_block_offset, ic_block) / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups / VECT_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(float v_mean[MAX_IC_BLOCK_SGROUPS];)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups; ++sg) {)==""\n"
R"==(v_mean[sg] = as_float(intel_sub_group_block_read()==""\n"
R"==((const __global uint *)(&mean[(sg * SUB_GROUP_SIZE)])));)==""\n"
R"==(})==""\n"
R"==(float diff_gamma[MAX_IC_BLOCK_SGROUPS] = {0.0f};)==""\n"
R"==(float diff_beta[MAX_IC_BLOCK_SGROUPS] = {0.0f};)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(for (int sg = 0; sg < ic_vect_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = sg * SUB_GROUP_SIZE * VECT_SIZE;)==""\n"
R"==(const int sgv = sg * VECT_SIZE;)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(const VECT_CHAR_T ws_vect = LOAD_VECT_CHAR(&ws[sg_idx]);)==""\n"
R"==(#endif)==""\n"
R"==(float src_vect[VECT_SIZE];)==""\n"
R"==(AS_VECT_FLOAT(src_vect) = LOAD_VECT_DATA(&src[sg_idx]);)==""\n"
R"==(VECT_FLOAT_T dd_vect = LOAD_VECT_DATA(&diff_dst[sg_idx]);)==""\n"
R"==(float v0[VECT_SIZE];)==""\n"
R"==(for (int vect = 0; vect < VECT_SIZE; ++vect) {)==""\n"
R"==(int sg_idx = sg * VECT_SIZE + vect;)==""\n"
R"==(v0[vect] = src_vect[vect] - v_mean[sg_idx];)==""\n"
R"==(})==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(dd_vect = select()==""\n"
R"==((VECT_FLOAT_T)0.0f, dd_vect, CONVERT_VECT_INT_T(ws_vect));)==""\n"
R"==(#endif)==""\n"
R"==(AS_VECT_FLOAT(&diff_gamma[sgv]) = fma(AS_VECT_FLOAT(v0), dd_vect,)==""\n"
R"==(AS_VECT_FLOAT(&diff_gamma[sgv]));)==""\n"
R"==(AS_VECT_FLOAT(&diff_beta[sgv]) += dd_vect;)==""\n"
R"==(})==""\n"
R"==(#if MAY_HAVE_IC_TAIL)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = ic_vect_sgroups * VECT_SIZE + sg;)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(char ws_tail = LOAD_CHAR_1x16(&ws[sg_idx * SUB_GROUP_SIZE]);)==""\n"
R"==(#endif)==""\n"
R"==(float src_tail = LOAD_DATA_1x16(&src[sg_idx * SUB_GROUP_SIZE]);)==""\n"
R"==(float dd_tail = LOAD_DATA_1x16(&diff_dst[sg_idx * SUB_GROUP_SIZE]);)==""\n"
R"==(float v0 = src_tail - v_mean[sg_idx];)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(dd_tail = select(0.0f, dd_tail, convert_int(ws_tail));)==""\n"
R"==(#endif)==""\n"
R"==(diff_gamma[sg_idx] = fma(v0, dd_tail, diff_gamma[sg_idx]);)==""\n"
R"==(diff_beta[sg_idx] += dd_tail;)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(src += ic_size;)==""\n"
R"==(diff_dst += ic_size;)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(ws += ic_size;)==""\n"
R"==(#endif)==""\n"
R"==(} )==""\n"
R"==(if (use_fused_atomics_reduction) {)==""\n"
R"==(nhwc_reusable_bwd_fused_reduction_buff(diff_scale, diff_shift,)==""\n"
R"==(ic_block_offset, (float *)(&diff_gamma), (float *)(&diff_beta),)==""\n"
R"==(local_sums, ic_block, calc_slm_size);)==""\n"
R"==(} else {)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups; ++sg) {)==""\n"
R"==(const int sg_off = sg * SUB_GROUP_SIZE;)==""\n"
R"==(STORE_FLOAT_1x16(&temp_reduce[sg_off], diff_gamma[sg]);)==""\n"
R"==(STORE_FLOAT_1x16(&temp_reduce_shift[sg_off], diff_beta[sg]);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_norm_bwd(__global DATA_T *src, __global float *mean,)==""\n"
R"==(__global float *variance, __global DATA_T *diff_dst,)==""\n"
R"==(__global float *scaleshift, __global char *ws,)==""\n"
R"==(__global DATA_T *diff_src, __global float *diff_scale,)==""\n"
R"==(__global float *diff_shift, float eps, __global DATA_T *diff_src_add,)==""\n"
R"==(off_t ic_size, off_t ic_block, off_t sp_size, off_t update_sp_block) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(variance += ic_block_offset;)==""\n"
R"==(mean += ic_block_offset;)==""\n"
R"==(diff_scale += ic_block_offset;)==""\n"
R"==(diff_shift += ic_block_offset;)==""\n"
R"==(scaleshift += ic_block_offset;)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int offset)==""\n"
R"==(= ic_block_offset + sp_block_idx * update_sp_block * ic_size;)==""\n"
R"==(src += offset;)==""\n"
R"==(diff_dst += offset;)==""\n"
R"==(ws += offset;)==""\n"
R"==(diff_src += offset;)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(diff_src_add += offset;)==""\n"
R"==(#endif)==""\n"
R"==(const bool has_sp_block_tail = sp_size % update_sp_block;)==""\n"
R"==(const int sp_idx_bnd = has_sp_block_tail)==""\n"
R"==(? min(update_sp_block, sp_size - sp_block_idx * update_sp_block))==""\n"
R"==(: update_sp_block;)==""\n"
R"==(const int ic_block_sgroups = ic_block / SUB_GROUP_SIZE;)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(for (int sg = 0; sg < ic_block_sgroups / VECT_SIZE; ++sg) {)==""\n"
R"==(const int sg_idx = sg * SUB_GROUP_SIZE * VECT_SIZE;)==""\n"
R"==(const VECT_FLOAT_T v_variance = LOAD_VECT_FLOAT(&variance[sg_idx]);)==""\n"
R"==(const VECT_FLOAT_T sqrt_variance)==""\n"
R"==(= (VECT_FLOAT_T)1.0f / sqrt(v_variance + (VECT_FLOAT_T)eps);)==""\n"
R"==(const VECT_FLOAT_T gamma = USE_SCALE)==""\n"
R"==(? LOAD_VECT_FLOAT(&scaleshift[sg_idx]))==""\n"
R"==(: (VECT_FLOAT_T)1.0f;)==""\n"
R"==(const VECT_FLOAT_T src_vect = LOAD_VECT_DATA(&src[sg_idx]);)==""\n"
R"==(VECT_FLOAT_T dd_vect = LOAD_VECT_DATA(&diff_dst[sg_idx]);)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(const VECT_CHAR_T ws_vect = LOAD_VECT_CHAR(&ws[sg_idx]);)==""\n"
R"==(dd_vect = select()==""\n"
R"==((VECT_FLOAT_T)0.0f, dd_vect, CONVERT_VECT_INT_T(ws_vect));)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(STORE_VECT_DATA(&diff_src_add[sg_idx], dd_vect);)==""\n"
R"==(#endif)==""\n"
R"==(#endif)==""\n"
R"==(#if CALCULATE_STATS == 1)==""\n"
R"==(const VECT_FLOAT_T v_mean = LOAD_VECT_FLOAT(&mean[sg_idx]);)==""\n"
R"==(const VECT_FLOAT_T diff_gamma)==""\n"
R"==(= LOAD_VECT_FLOAT(&diff_scale[sg_idx]);)==""\n"
R"==(const VECT_FLOAT_T diff_beta = LOAD_VECT_FLOAT(&diff_shift[sg_idx]);)==""\n"
R"==(dd_vect -= (diff_beta)==""\n"
R"==(+ (src_vect - v_mean) * diff_gamma)==""\n"
R"==(* sqrt_variance))==""\n"
R"==(/ sp_size;)==""\n"
R"==(#endif)==""\n"
R"==(dd_vect *= gamma * sqrt_variance;)==""\n"
R"==(STORE_VECT_DATA(&diff_src[sg_idx], dd_vect);)==""\n"
R"==(} )==""\n"
R"==(const int ic_tail_sgroups = (ic_block / SUB_GROUP_SIZE) % VECT_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups - ic_tail_sgroups;)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = (ic_vect_sgroups + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(const float v_variance = LOAD_FLOAT_1x16(&variance[sg_idx]);)==""\n"
R"==(const float sqrt_variance)==""\n"
R"==(= (float)1.0f / sqrt(v_variance + (float)eps);)==""\n"
R"==(const float gamma)==""\n"
R"==(= USE_SCALE ? LOAD_FLOAT_1x16(&scaleshift[sg_idx]) : 1.0f;)==""\n"
R"==(const float src_vect = LOAD_DATA_1x16(&src[sg_idx]);)==""\n"
R"==(float dd_vect = LOAD_DATA_1x16(&diff_dst[sg_idx]);)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(const char ws_vect = LOAD_CHAR_1x16(&ws[sg_idx]);)==""\n"
R"==(dd_vect = select(0.0f, dd_vect, convert_int(ws_vect));)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(STORE_DATA_1x16(&diff_src_add[sg_idx], dd_vect);)==""\n"
R"==(#endif)==""\n"
R"==(#endif)==""\n"
R"==(#if CALCULATE_STATS == 1)==""\n"
R"==(const float v_mean = LOAD_FLOAT_1x16(&mean[sg_idx]);)==""\n"
R"==(const float diff_gamma = LOAD_FLOAT_1x16(&diff_scale[sg_idx]);)==""\n"
R"==(const float diff_beta = LOAD_FLOAT_1x16(&diff_shift[sg_idx]);)==""\n"
R"==(dd_vect -= (diff_beta)==""\n"
R"==(+ (src_vect - v_mean) * diff_gamma)==""\n"
R"==(* sqrt_variance))==""\n"
R"==(/ sp_size;)==""\n"
R"==(#endif)==""\n"
R"==(dd_vect *= gamma * sqrt_variance;)==""\n"
R"==(STORE_DATA_1x16(&diff_src[sg_idx], dd_vect);)==""\n"
R"==(})==""\n"
R"==(src += ic_size;)==""\n"
R"==(diff_dst += ic_size;)==""\n"
R"==(diff_src += ic_size;)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(diff_src_add += ic_size;)==""\n"
R"==(#endif)==""\n"
R"==(ws += ic_size;)==""\n"
R"==(#endif)==""\n"
R"==(} )==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_norm_bwd_buff(__global DATA_T *src, __global float *mean,)==""\n"
R"==(__global float *variance, __global DATA_T *diff_dst,)==""\n"
R"==(__global float *scaleshift, __global char *ws,)==""\n"
R"==(__global DATA_T *diff_src, __global float *diff_scale,)==""\n"
R"==(__global float *diff_shift, float eps, __global DATA_T *diff_src_add,)==""\n"
R"==(off_t ic_size, off_t ic_block, off_t sp_size, off_t update_sp_block) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(const int ic_block_offset = (c / SUB_GROUP_SIZE) * ic_block;)==""\n"
R"==(variance += ic_block_offset;)==""\n"
R"==(mean += ic_block_offset;)==""\n"
R"==(diff_scale += ic_block_offset;)==""\n"
R"==(diff_shift += ic_block_offset;)==""\n"
R"==(scaleshift += ic_block_offset;)==""\n"
R"==(const int sp_block_idx = get_global_id(1);)==""\n"
R"==(const int offset)==""\n"
R"==(= ic_block_offset + sp_block_idx * update_sp_block * ic_size;)==""\n"
R"==(src += offset;)==""\n"
R"==(diff_dst += offset;)==""\n"
R"==(ws += offset;)==""\n"
R"==(diff_src += offset;)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(diff_src_add += offset;)==""\n"
R"==(#endif)==""\n"
R"==(const bool has_sp_block_tail = sp_size % update_sp_block;)==""\n"
R"==(const int sp_idx_bnd = has_sp_block_tail)==""\n"
R"==(? min(update_sp_block, sp_size - sp_block_idx * update_sp_block))==""\n"
R"==(: update_sp_block;)==""\n"
R"==(const int ic_block_sgroups)==""\n"
R"==(= min(ic_size - ic_block_offset, ic_block) / SUB_GROUP_SIZE;)==""\n"
R"==(const int ic_vect_sgroups = ic_block_sgroups / VECT_SIZE;)==""\n"
R"==(const int ic_tail_sgroups = ic_block_sgroups % VECT_SIZE;)==""\n"
R"==(float v_variance[MAX_IC_BLOCK_SGROUPS], v_mean[MAX_IC_BLOCK_SGROUPS],)==""\n"
R"==(diff_gamma[MAX_IC_BLOCK_SGROUPS], diff_beta[MAX_IC_BLOCK_SGROUPS],)==""\n"
R"==(sqrt_variance[MAX_IC_BLOCK_SGROUPS], gamma[MAX_IC_BLOCK_SGROUPS];)==""\n"
R"==(for (int sg = 0; sg < ic_vect_sgroups; ++sg) {)==""\n"
R"==(const int sgv = sg * VECT_SIZE;)==""\n"
R"==(const int sg_idx = sg * SUB_GROUP_SIZE * VECT_SIZE;)==""\n"
R"==(AS_VECT_FLOAT(&v_variance[sgv]) = LOAD_VECT_FLOAT(&variance[sg_idx]);)==""\n"
R"==(#if CALCULATE_STATS == 1)==""\n"
R"==(AS_VECT_FLOAT(&v_mean[sgv]) = LOAD_VECT_FLOAT(&mean[sg_idx]);)==""\n"
R"==(AS_VECT_FLOAT(&diff_gamma[sgv]) = LOAD_VECT_FLOAT(&diff_scale[sg_idx]);)==""\n"
R"==(AS_VECT_FLOAT(&diff_beta[sgv]) = LOAD_VECT_FLOAT(&diff_shift[sg_idx]);)==""\n"
R"==(#endif )==""\n"
R"==(AS_VECT_FLOAT(&gamma[sgv]) = USE_SCALE)==""\n"
R"==(? LOAD_VECT_FLOAT(&scaleshift[sg_idx]))==""\n"
R"==(: (VECT_FLOAT_T)1.0f;)==""\n"
R"==(AS_VECT_FLOAT(&sqrt_variance[sgv]) = (VECT_FLOAT_T)1.0f)==""\n"
R"==(/ sqrt(AS_VECT_FLOAT(&v_variance[sgv]) + (VECT_FLOAT_T)eps);)==""\n"
R"==(})==""\n"
R"==(#if MAY_HAVE_IC_TAIL)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sgv = ic_vect_sgroups * VECT_SIZE + sg;)==""\n"
R"==(const int sg_idx = (ic_vect_sgroups * VECT_SIZE + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(v_variance[sgv] = LOAD_FLOAT_1x16(&variance[sg_idx]);)==""\n"
R"==(#if CALCULATE_STATS == 1)==""\n"
R"==(v_mean[sgv] = LOAD_FLOAT_1x16(&mean[sg_idx]);)==""\n"
R"==(diff_gamma[sgv] = LOAD_FLOAT_1x16(&diff_scale[sg_idx]);)==""\n"
R"==(diff_beta[sgv] = LOAD_FLOAT_1x16(&diff_shift[sg_idx]);)==""\n"
R"==(#endif )==""\n"
R"==(gamma[sgv] = USE_SCALE ? LOAD_FLOAT_1x16(&scaleshift[sg_idx]) : 1.0f;)==""\n"
R"==(sqrt_variance[sgv] = 1.0f / sqrt(v_variance[sgv] + eps);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(for (int sp = 0; sp < sp_idx_bnd; ++sp) {)==""\n"
R"==(for (int sg = 0; sg < ic_vect_sgroups; ++sg) {)==""\n"
R"==(const int sg_idx = sg * SUB_GROUP_SIZE * VECT_SIZE;)==""\n"
R"==(const int sgv = sg * VECT_SIZE;)==""\n"
R"==(const VECT_FLOAT_T src_vect = LOAD_VECT_DATA(&src[sg_idx]);)==""\n"
R"==(VECT_FLOAT_T dd_vect = LOAD_VECT_DATA(&diff_dst[sg_idx]);)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(const VECT_CHAR_T ws_vect = LOAD_VECT_CHAR(&ws[sg_idx]);)==""\n"
R"==(dd_vect = select()==""\n"
R"==((VECT_FLOAT_T)0.0f, dd_vect, CONVERT_VECT_INT_T(ws_vect));)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(STORE_VECT_DATA(&diff_src_add[sg_idx], dd_vect);)==""\n"
R"==(#endif)==""\n"
R"==(#endif)==""\n"
R"==(#if CALCULATE_STATS == 1)==""\n"
R"==(dd_vect -= (AS_VECT_FLOAT(&diff_beta[sgv]))==""\n"
R"==(+ (src_vect - AS_VECT_FLOAT(&v_mean[sgv])))==""\n"
R"==(* AS_VECT_FLOAT(&diff_gamma[sgv]))==""\n"
R"==(* AS_VECT_FLOAT(&sqrt_variance[sgv])))==""\n"
R"==(/ sp_size;)==""\n"
R"==(#endif)==""\n"
R"==(dd_vect *= AS_VECT_FLOAT(&gamma[sgv]))==""\n"
R"==(* AS_VECT_FLOAT(&sqrt_variance[sgv]);)==""\n"
R"==(STORE_VECT_DATA(&diff_src[sg_idx], dd_vect);)==""\n"
R"==(} )==""\n"
R"==(#if MAY_HAVE_IC_TAIL)==""\n"
R"==(for (int sg = 0; sg < ic_tail_sgroups; ++sg) {)==""\n"
R"==(const int sgv = ic_vect_sgroups * VECT_SIZE + sg;)==""\n"
R"==(const int sg_idx)==""\n"
R"==(= (ic_vect_sgroups * VECT_SIZE + sg) * SUB_GROUP_SIZE;)==""\n"
R"==(const float src_tail = LOAD_DATA_1x16(&src[sg_idx]);)==""\n"
R"==(float dd_tail = LOAD_DATA_1x16(&diff_dst[sg_idx]);)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(const char ws_tail = LOAD_CHAR_1x16(&ws[sg_idx]);)==""\n"
R"==(dd_tail = select(0.0f, dd_tail, convert_int(ws_tail));)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(STORE_DATA_1x16(&diff_src_add[sg_idx], dd_tail);)==""\n"
R"==(#endif)==""\n"
R"==(#endif)==""\n"
R"==(#if CALCULATE_STATS == 1)==""\n"
R"==(dd_tail -= (diff_beta[sgv])==""\n"
R"==(+ (src_tail - v_mean[sgv]) * diff_gamma[sgv])==""\n"
R"==(* sqrt_variance[sgv]))==""\n"
R"==(/ sp_size;)==""\n"
R"==(#endif)==""\n"
R"==(dd_tail *= gamma[sgv] * sqrt_variance[sgv];)==""\n"
R"==(STORE_DATA_1x16(&diff_src[sg_idx], dd_tail);)==""\n"
R"==(} )==""\n"
R"==(#endif)==""\n"
R"==(src += ic_size;)==""\n"
R"==(diff_dst += ic_size;)==""\n"
R"==(diff_src += ic_size;)==""\n"
R"==(#if FUSE_BN_RELU)==""\n"
R"==(#if FUSE_BN_ADD_RELU)==""\n"
R"==(diff_src_add += ic_size;)==""\n"
R"==(#endif)==""\n"
R"==(ws += ic_size;)==""\n"
R"==(#endif)==""\n"
R"==(} )==""\n"
R"==(})==""\n"
R"==(__kernel void nhwc_reusable_reduce_aux(__global float *ptr1,)==""\n"
R"==(__global float *ptr2, float eps, off_t sp_size, int use_stats_one_pass,)==""\n"
R"==(int init_stage, int is_fwd) {)==""\n"
R"==(const int c = get_global_id(0);)==""\n"
R"==(if (init_stage) {)==""\n"
R"==(ptr1[c] = 0.0f;)==""\n"
R"==(ptr2[c] = 0.0f;)==""\n"
R"==(} else {)==""\n"
R"==(if (is_fwd) {)==""\n"
R"==(if (use_stats_one_pass) {)==""\n"
R"==(ptr1[c] /= sp_size;)==""\n"
R"==(float tmp_var)==""\n"
R"==(= max(0.0f, (ptr2[c] / sp_size) - (ptr1[c] * ptr1[c]));)==""\n"
R"==(ptr2[c] = tmp_var;)==""\n"
R"==(} else {)==""\n"
R"==(ptr1[c] /= sp_size;)==""\n"
R"==(})==""\n"
R"==(} else {)==""\n"
R"==(ptr1[c] *= 1.0f / sqrt(ptr2[c] + eps);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_reduce_fwd_reg(__global float *reduce_scratchpad,)==""\n"
R"==(off_t scratchpad_off, __global float *dst, off_t ic_size,)==""\n"
R"==(off_t reduce_ic_sub_groups, off_t reduce_stat_nblocks, off_t sp_size,)==""\n"
R"==(__local float *local_sum) {)==""\n"
R"==(const int ic_sub_group = get_global_id(0) / SUB_GROUP_SIZE;)==""\n"
R"==(const int group_c = get_global_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int c = group_c * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(float sum = 0.0f;)==""\n"
R"==(const int reduce_chunk = reduce_stat_nblocks / reduce_ic_sub_groups;)==""\n"
R"==(const int reduce_scratchpad_off)==""\n"
R"==(= scratchpad_off + c + ic_sub_group * reduce_chunk * ic_size;)==""\n"
R"==(reduce_scratchpad += reduce_scratchpad_off;)==""\n"
R"==(unroll_16_for(int i = 0; i < reduce_chunk; i++) {)==""\n"
R"==(sum += reduce_scratchpad[i * ic_size];)==""\n"
R"==(})==""\n"
R"==(if (ic_sub_group > 0) {)==""\n"
R"==(local_sum[ic_sub_group * SUB_GROUP_SIZE + simd_id] = sum;)==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (ic_sub_group == 0) {)==""\n"
R"==(unroll_16_for(int i = 1; i < reduce_ic_sub_groups; i++) {)==""\n"
R"==(sum += local_sum[i * SUB_GROUP_SIZE + simd_id];)==""\n"
R"==(})==""\n"
R"==(dst[c] = sum / sp_size;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_reduce_fwd_1pass(__global float *reduce_temp,)==""\n"
R"==(__global float *mean, __global float *variance, off_t ic_size,)==""\n"
R"==(off_t reduce_ic_sub_groups, off_t reduce_stat_nblocks, off_t sp_size,)==""\n"
R"==(__local SUM_DATA_T *local_sum, __local SUM_DATA_T *local_sum_sq) {)==""\n"
R"==(const int ic_sub_group = get_global_id(0) / SUB_GROUP_SIZE;)==""\n"
R"==(const int group_c = get_global_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int c = group_c * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(SUM_DATA_T sum;)==""\n"
R"==(SUM_DATA_T sum_sq;)==""\n"
R"==(sum.s0 = 0;)==""\n"
R"==(sum.s1 = 0;)==""\n"
R"==(sum_sq.s0 = 0;)==""\n"
R"==(sum_sq.s1 = 0;)==""\n"
R"==(const int offs_sq = reduce_stat_nblocks * ic_size;)==""\n"
R"==(const int reduce_chunk = reduce_stat_nblocks / reduce_ic_sub_groups;)==""\n"
R"==(const int offs = c + ic_sub_group * reduce_chunk * ic_size;)==""\n"
R"==(unroll_16_for(int i = 0; i < reduce_chunk; i++) {)==""\n"
R"==(float tmp = reduce_temp[offs + i * ic_size];)==""\n"
R"==(sum = summation(tmp, sum);)==""\n"
R"==(})==""\n"
R"==(unroll_16_for(int i = 0; i < reduce_chunk; i++) {)==""\n"
R"==(float tmp = reduce_temp[offs_sq + offs + i * ic_size];)==""\n"
R"==(sum_sq = summation(tmp, sum_sq);)==""\n"
R"==(})==""\n"
R"==(if (ic_sub_group > 0) {)==""\n"
R"==(local_sum[ic_sub_group * SUB_GROUP_SIZE + simd_id] = sum;)==""\n"
R"==(local_sum_sq[ic_sub_group * SUB_GROUP_SIZE + simd_id] = sum_sq;)==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (ic_sub_group == 0) {)==""\n"
R"==(unroll_16_for(int i = 1; i < reduce_ic_sub_groups; i++) {)==""\n"
R"==(SUM_DATA_T tmp = local_sum[i * SUB_GROUP_SIZE + simd_id];)==""\n"
R"==(SUM_DATA_T tmp_sq = local_sum_sq[i * SUB_GROUP_SIZE + simd_id];)==""\n"
R"==(sum = summation(tmp.s1, sum);)==""\n"
R"==(sum_sq = summation(tmp_sq.s1, sum_sq);)==""\n"
R"==(sum = summation(tmp.s0, sum);)==""\n"
R"==(sum_sq = summation(tmp_sq.s0, sum_sq);)==""\n"
R"==(})==""\n"
R"==(float tmp_mean = sum.s0 / sp_size;)==""\n"
R"==(mean[c] = tmp_mean;)==""\n"
R"==(float tmp_var)==""\n"
R"==(= max(0.0f, (sum_sq.s0 / sp_size) - (tmp_mean * tmp_mean));)==""\n"
R"==(variance[c] = tmp_var;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SUB_GROUP_SIZE))) __kernel void)==""\n"
R"==(nhwc_reusable_reduce_stat(__global float *temp_reduce,)==""\n"
R"==(__global float *temp_reduce_shift, __global float *diff_scale,)==""\n"
R"==(__global float *diff_shift, __global float *variance, float eps,)==""\n"
R"==(off_t ic_size, off_t reduce_ic_sub_groups, off_t reduce_stat_nblocks,)==""\n"
R"==(__local float *local_gamma, __local float *local_beta) {)==""\n"
R"==(const int ic_sub_group = get_global_id(0) / SUB_GROUP_SIZE;)==""\n"
R"==(const int group_c = get_global_id(1);)==""\n"
R"==(const int simd_id = get_sub_group_local_id();)==""\n"
R"==(const int c = group_c * SUB_GROUP_SIZE + simd_id;)==""\n"
R"==(float diff_gamma = 0.0f;)==""\n"
R"==(float diff_beta = 0.0f;)==""\n"
R"==(const int reduce_chunk = reduce_stat_nblocks / reduce_ic_sub_groups;)==""\n"
R"==(const int scratchpad_off)==""\n"
R"==(= ic_size + c + ic_sub_group * reduce_chunk * ic_size;)==""\n"
R"==(temp_reduce += scratchpad_off;)==""\n"
R"==(temp_reduce_shift += scratchpad_off;)==""\n"
R"==(unroll_16_for(int i = 0; i < reduce_chunk; i++) {)==""\n"
R"==(diff_gamma += temp_reduce[i * ic_size];)==""\n"
R"==(diff_beta += temp_reduce_shift[i * ic_size];)==""\n"
R"==(})==""\n"
R"==(if (ic_sub_group > 0) {)==""\n"
R"==(local_gamma[ic_sub_group * SUB_GROUP_SIZE + simd_id] = diff_gamma;)==""\n"
R"==(local_beta[ic_sub_group * SUB_GROUP_SIZE + simd_id] = diff_beta;)==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (ic_sub_group == 0) {)==""\n"
R"==(unroll_16_for(int i = 1; i < reduce_ic_sub_groups; i++) {)==""\n"
R"==(diff_gamma += local_gamma[i * SUB_GROUP_SIZE + simd_id];)==""\n"
R"==(diff_beta += local_beta[i * SUB_GROUP_SIZE + simd_id];)==""\n"
R"==(})==""\n"
R"==(float sqrt_variance = 1.0f / sqrt(variance[c] + eps);)==""\n"
R"==(diff_scale[c] = diff_gamma * sqrt_variance;)==""\n"
R"==(diff_shift[c] = diff_beta;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==()==";
}
}
}
}
}