namespace dnnl {
namespace impl {
namespace gpu {
namespace intel {
namespace ocl {
const char *vectorized_resampling_kernel = R"==(/*******************************************************************************)==""\n"
R"==(* Copyright 2022-2024 Intel Corporation)==""\n"
R"==(*)==""\n"
R"==(* Licensed under the Apache License, Version 2.0 (the "License");)==""\n"
R"==(* you may not use this file except in compliance with the License.)==""\n"
R"==(* You may obtain a copy of the License at)==""\n"
R"==(*)==""\n"
R"==(* http:)==""\n"
R"==(*)==""\n"
R"==(* Unless required by applicable law or agreed to in writing, software)==""\n"
R"==(* distributed under the License is distributed on an "AS IS" BASIS,)==""\n"
R"==(* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.)==""\n"
R"==(* See the License for the specific language governing permissions and)==""\n"
R"==(* limitations under the License.)==""\n"
R"==(*******************************************************************************/)==""\n"
R"==(#include "gpu/intel/ocl/dispatch.h")==""\n"
R"==(#include "gpu/intel/ocl/ocl_post_ops.h")==""\n"
R"==(#include "gpu/intel/ocl/ocl_types.h")==""\n"
R"==(#if IS_BWD == 1)==""\n"
R"==(inline float linear(int x, int fo, int fi) {)==""\n"
R"==(return ((x + .5f) * fo / fi) - .5f;)==""\n"
R"==(})==""\n"
R"==(inline int ceil_pos(float x) {)==""\n"
R"==(return max((int)ceil(x), (int)0);)==""\n"
R"==(})==""\n"
R"==(#define DST_MB_STRIDE(x) (x % DST_B0) * DST_SB0 + (x / DST_B0) * DST_S0)==""\n"
R"==(#define DST_C_STRIDE(x) (x % DST_B1) * DST_SB1 + (x / DST_B1) * DST_S1)==""\n"
R"==(#if NDIMS == 3)==""\n"
R"==(#define OW_STRIDE(x) (x % DST_B2) * DST_SB2 + (x / DST_B2) * DST_S2)==""\n"
R"==(#define OH_STRIDE(x) 0)==""\n"
R"==(#define OD_STRIDE(x) 0)==""\n"
R"==(#elif NDIMS == 4)==""\n"
R"==(#define OW_STRIDE(x) (x % DST_B3) * DST_SB3 + (x / DST_B3) * DST_S3)==""\n"
R"==(#define OH_STRIDE(x) (x % DST_B2) * DST_SB2 + (x / DST_B2) * DST_S2)==""\n"
R"==(#define OD_STRIDE(x) 0)==""\n"
R"==(#elif NDIMS == 5)==""\n"
R"==(#define OW_STRIDE(x) (x % DST_B4) * DST_SB4 + (x / DST_B4) * DST_S4)==""\n"
R"==(#define OH_STRIDE(x) (x % DST_B3) * DST_SB3 + (x / DST_B3) * DST_S3)==""\n"
R"==(#define OD_STRIDE(x) (x % DST_B2) * DST_SB2 + (x / DST_B2) * DST_S2)==""\n"
R"==(#endif)==""\n"
R"==(KERNEL_ATTR)==""\n"
R"==(__kernel void vectorized_resampling_bwd()==""\n"
R"==(__global DST_DATA_T *diff_src, __global const DATA_T *diff_dst) {)==""\n"
R"==(const uint sglid = get_sub_group_local_id();)==""\n"
R"==(const uint mb = (get_global_id(0) / MB_STRIDE);)==""\n"
R"==(const uint c_start = (get_global_id(0) / GWS_SGS_DEFAULT) * GWS_SGS_DEFAULT)==""\n"
R"==(* VECT_DT_N % PADDED_C;)==""\n"
R"==(const uint c = c_start + sglid;)==""\n"
R"==(const uint id = (get_global_id(0) / ID_STRIDE) % ID;)==""\n"
R"==(const uint ih = (get_global_id(0) / IH_STRIDE) % IH;)==""\n"
R"==(const uint iw = (get_global_id(0) / IW_STRIDE) % IW;)==""\n"
R"==(const uint src_index = SRC_OFF(mb, c_start, id, ih, iw);)==""\n"
R"==(VECT_DEF_ACC_DATA_T src_val = 0.0f;)==""\n"
R"==(#if RESAMPLING_ALG_NEAREST)==""\n"
R"==(if (mb >= MB || c >= C) return;)==""\n"
R"==(int od_start = ceil_pos(id * FD - .5f);)==""\n"
R"==(int oh_start = ceil_pos(ih * FH - .5f);)==""\n"
R"==(int ow_start = ceil_pos(iw * FW - .5f);)==""\n"
R"==(int od_end = ceil_pos((id + 1.f) * FD - .5f);)==""\n"
R"==(int oh_end = ceil_pos((ih + 1.f) * FH - .5f);)==""\n"
R"==(int ow_end = ceil_pos((iw + 1.f) * FW - .5f);)==""\n"
R"==(for_(int i = od_start; i < od_end; i++))==""\n"
R"==(for_(int j = oh_start; j < oh_end; j++))==""\n"
R"==(for (int k = ow_start; k < ow_end; k++) {)==""\n"
R"==(const int dst_index = DST_OFF(mb, c_start, i, j, k);)==""\n"
R"==(#if VECT_DT_N == 1)==""\n"
R"==(src_val += AS_VECT_DEF_ACC_DATA_T(diff_dst[dst_index + sglid]);)==""\n"
R"==(#else)==""\n"
R"==(for (int i = 0; i < VECT_DT_N && c + GWS_SGS_DEFAULT * i < C; i++) {)==""\n"
R"==(src_val[i] += TO_DEF_ACC_DATA_T()==""\n"
R"==(diff_dst[dst_index + sglid + GWS_SGS_DEFAULT * i]);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(})==""\n"
R"==(#else)==""\n"
R"==(int my_idx;)==""\n"
R"==({)==""\n"
R"==(int ix, OX, IX;)==""\n"
R"==(if (sglid < 4) {)==""\n"
R"==(ix = id;)==""\n"
R"==(OX = OD;)==""\n"
R"==(IX = ID;)==""\n"
R"==(} else if (sglid < 8) {)==""\n"
R"==(ix = ih;)==""\n"
R"==(OX = OH;)==""\n"
R"==(IX = IH;)==""\n"
R"==(} else {)==""\n"
R"==(ix = iw;)==""\n"
R"==(OX = OW;)==""\n"
R"==(IX = IW;)==""\n"
R"==(})==""\n"
R"==(if (sglid % 4 == 1) { ix -= 1; })==""\n"
R"==(if (sglid % 4 == 2) { ix += 1; })==""\n"
R"==(float idx_intermediate = linear(ix, OX, IX);)==""\n"
R"==(if (sglid % 2 == 0) {)==""\n"
R"==(my_idx = ceil_pos(idx_intermediate);)==""\n"
R"==(} else {)==""\n"
R"==(my_idx = (idx_intermediate < 0) ? 0 : (int)idx_intermediate + 1;)==""\n"
R"==(})==""\n"
R"==(if (sglid % 4 >= 2) { my_idx = min(my_idx, OX); })==""\n"
R"==(if (sglid % 4 == 0 && ix == 0) { my_idx = 0; })==""\n"
R"==(if (sglid % 4 == 3 && ix == IX - 1) { my_idx = OX; })==""\n"
R"==(})==""\n"
R"==(int od_start[2])==""\n"
R"==(= {sub_group_shuffle(my_idx, 0), sub_group_shuffle(my_idx, 1)};)==""\n"
R"==(int od_end[2])==""\n"
R"==(= {sub_group_shuffle(my_idx, 2), sub_group_shuffle(my_idx, 3)};)==""\n"
R"==(int oh_start[2])==""\n"
R"==(= {sub_group_shuffle(my_idx, 4), sub_group_shuffle(my_idx, 5)};)==""\n"
R"==(int oh_end[2])==""\n"
R"==(= {sub_group_shuffle(my_idx, 6), sub_group_shuffle(my_idx, 7)};)==""\n"
R"==(int ow_start[2])==""\n"
R"==(= {sub_group_shuffle(my_idx, 8), sub_group_shuffle(my_idx, 9)};)==""\n"
R"==(int ow_end[2])==""\n"
R"==(= {sub_group_shuffle(my_idx, 10), sub_group_shuffle(my_idx, 11)};)==""\n"
R"==(const int num_od_left = od_end[0] - od_start[0];)==""\n"
R"==(const int num_od_right = od_end[1] - od_start[1];)==""\n"
R"==(const int num_oh_left = oh_end[0] - oh_start[0];)==""\n"
R"==(const int num_oh_right = oh_end[1] - oh_start[1];)==""\n"
R"==(const int num_ow_left = ow_end[0] - ow_start[0];)==""\n"
R"==(const int num_ow_right = ow_end[1] - ow_start[1];)==""\n"
R"==(float myres;)==""\n"
R"==({)==""\n"
R"==(int ox, IX, OX;)==""\n"
R"==(int offset = sglid;)==""\n"
R"==(if (0 <= offset && offset < num_od_left) {)==""\n"
R"==(OX = OD;)==""\n"
R"==(IX = ID;)==""\n"
R"==(ox = od_start[0] + offset;)==""\n"
R"==(})==""\n"
R"==(offset -= num_od_left;)==""\n"
R"==(if (0 <= offset && offset < num_od_right) {)==""\n"
R"==(OX = OD;)==""\n"
R"==(IX = ID;)==""\n"
R"==(ox = od_start[1] + offset;)==""\n"
R"==(})==""\n"
R"==(offset -= num_od_right;)==""\n"
R"==(if (0 <= offset && offset < num_oh_left) {)==""\n"
R"==(OX = OH;)==""\n"
R"==(IX = IH;)==""\n"
R"==(ox = oh_start[0] + offset;)==""\n"
R"==(})==""\n"
R"==(offset -= num_oh_left;)==""\n"
R"==(if (0 <= offset && offset < num_oh_right) {)==""\n"
R"==(OX = OH;)==""\n"
R"==(IX = IH;)==""\n"
R"==(ox = oh_start[1] + offset;)==""\n"
R"==(})==""\n"
R"==(offset -= num_oh_right;)==""\n"
R"==(if (0 <= offset && offset < num_ow_left) {)==""\n"
R"==(OX = OW;)==""\n"
R"==(IX = IW;)==""\n"
R"==(ox = ow_start[0] + offset;)==""\n"
R"==(})==""\n"
R"==(offset -= num_ow_left;)==""\n"
R"==(if (0 <= offset && offset < num_ow_right) {)==""\n"
R"==(OX = OW;)==""\n"
R"==(IX = IW;)==""\n"
R"==(ox = ow_start[1] + offset;)==""\n"
R"==(})==""\n"
R"==(const float x = linear(ox, IX, OX);)==""\n"
R"==(myres = fabs(x - trunc(x));)==""\n"
R"==(})==""\n"
R"==(float d_list[2][MAX_NUM_D];)==""\n"
R"==(float h_list[2][MAX_NUM_H];)==""\n"
R"==(float w_list[2][MAX_NUM_W];)==""\n"
R"==(for (int d = 0; d < num_od_left; d++) {)==""\n"
R"==(d_list[0][d] = 1.0f - sub_group_shuffle(myres, d);)==""\n"
R"==(})==""\n"
R"==(int offset = num_od_left;)==""\n"
R"==(for (int d = 0; d < num_od_right; d++) {)==""\n"
R"==(d_list[1][d] = sub_group_shuffle(myres, d + offset);)==""\n"
R"==(})==""\n"
R"==(offset += num_od_right;)==""\n"
R"==(for (int h = 0; h < num_oh_left; h++) {)==""\n"
R"==(h_list[0][h] = 1.0f - sub_group_shuffle(myres, h + offset);)==""\n"
R"==(})==""\n"
R"==(offset += num_oh_left;)==""\n"
R"==(for (int h = 0; h < num_oh_right; h++) {)==""\n"
R"==(h_list[1][h] = sub_group_shuffle(myres, h + offset);)==""\n"
R"==(})==""\n"
R"==(offset += num_oh_right;)==""\n"
R"==(for (int w = 0; w < num_ow_left; w++) {)==""\n"
R"==(w_list[0][w] = 1.0f - sub_group_shuffle(myres, w + offset);)==""\n"
R"==(})==""\n"
R"==(offset += num_ow_left;)==""\n"
R"==(for (int w = 0; w < num_ow_right; w++) {)==""\n"
R"==(w_list[1][w] = sub_group_shuffle(myres, w + offset);)==""\n"
R"==(})==""\n"
R"==(if (mb >= MB || c >= C) return;)==""\n"
R"==(const uint mb_c_off = DST_MB_STRIDE(mb) + DST_C_STRIDE(c_start);)==""\n"
R"==(for_(int c1 = 0; c1 < 2; c1++))==""\n"
R"==(for (int od = od_start[c1], i = 0; i < MAX_NUM_D && od < od_end[c1];)==""\n"
R"==(od++, i++) {)==""\n"
R"==(const uint d_off = mb_c_off + OD_STRIDE(od);)==""\n"
R"==(float Wid = d_list[c1][i];)==""\n"
R"==(for_(int c2 = 0; c2 < 2; c2++))==""\n"
R"==(for (int oh = oh_start[c2], j = 0; j < MAX_NUM_H && oh < oh_end[c2];)==""\n"
R"==(oh++, j++) {)==""\n"
R"==(const uint h_off = d_off + OH_STRIDE(oh);)==""\n"
R"==(float Wih = h_list[c2][j];)==""\n"
R"==(unroll_for(int c3 = 0; c3 < 2; c3++))==""\n"
R"==(unroll_for(int k = 0, ow = ow_start[c3];)==""\n"
R"==(k < MAX_NUM_W && ow < ow_end[c3]; k++, ow++) {)==""\n"
R"==(const uint dst_off = h_off + OW_STRIDE(ow);)==""\n"
R"==(#if VECT_DT_N == 1)==""\n"
R"==(VECT_DEF_ACC_DATA_T dst_val)==""\n"
R"==(= AS_VECT_DEF_ACC_DATA_T(diff_dst[dst_off + sglid]);)==""\n"
R"==(#else)==""\n"
R"==(VECT_DEF_ACC_DATA_T dst_val = 0;)==""\n"
R"==(for (int idx = 0;)==""\n"
R"==(idx < VECT_DT_N && c + GWS_SGS_DEFAULT * idx < C;)==""\n"
R"==(idx++) {)==""\n"
R"==(dst_val[idx] = TO_DEF_ACC_DATA_T()==""\n"
R"==(diff_dst[dst_off + sglid + GWS_SGS_DEFAULT * idx]);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(float Wiw = w_list[c3][k];)==""\n"
R"==(src_val += dst_val * Wid * Wih * Wiw;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(#if VECT_DT_N == 1)==""\n"
R"==(diff_src[src_index + sglid] = TO_DST(src_val);)==""\n"
R"==(#else)==""\n"
R"==(for (int i = 0; i < VECT_DT_N && c + GWS_SGS_DEFAULT * i < C; i++) {)==""\n"
R"==(diff_src[src_index + sglid + GWS_SGS_DEFAULT * i] = TO_DST(src_val[i]);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==()==";
}
}
}
}
}